package net.wanji.web.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.web.bo.AddSpecialServiceBO;
import net.wanji.web.bo.IsValidPointBO;
import net.wanji.web.bo.SpecialServiceRouteBO;
import net.wanji.web.service.SpecialServiceService;
import net.wanji.web.vo.RouteElementVO;
import net.wanji.web.vo.specialService.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.ws.rs.core.MediaType;
import java.util.List;
import java.util.Set;

/**
 * 快速特勤
 *
 * @author Kent HAN
 * @date 2022/11/8 10:57
 */
@Api(value = "SpecialServiceController", description = "特勤管理")
@RequestMapping("/specialService")
@RestController
public class SpecialServiceController {
    @Autowired
    SpecialServiceService specialServiceService;

    @AspectLog(description = "查询地图点位是否可选", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "查询地图点位是否可选", notes = "查询地图点位是否可选", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/isValidPoint",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject isValidPoint(@RequestBody IsValidPointBO isValidPointBO) {
        specialServiceService.isValidPoint(isValidPointBO);
        return JsonViewObject.newInstance().success();
    }

    @AspectLog(description = "备选路线推送", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "备选路线推送", notes = "备选路线推送", response = RouteElementVO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/specialServiceRoute",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject specialServiceRoute(@RequestBody SpecialServiceRouteBO specialServiceRouteBO) {
        List<List<RouteElementVO>> res = specialServiceService.specialServiceRoute(specialServiceRouteBO);
        return JsonViewObject.newInstance().success(res);
    }

    @AspectLog(description = "添加特勤", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "添加特勤", notes = "添加特勤", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/addSpecialService",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject addSpecialService(@RequestBody @Valid AddSpecialServiceBO addSpecialServiceBO) {
        specialServiceService.addSpecialService(addSpecialServiceBO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }

    @AspectLog(description = "快速特勤列表", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "快速特勤列表", notes = "快速特勤列表", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/listSpecialServices",
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = ListSpecialServicesVO.class),
    })
    public JsonViewObject listSpecialServices() {
        List<ListSpecialServicesVO> listSpecialServicesVOList = specialServiceService.listSpecialServices();
        return JsonViewObject.newInstance().success(listSpecialServicesVOList);
    }

    @AspectLog(description = "编辑特勤", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "编辑特勤", notes = "编辑特勤", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/updateSpecialService",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject updateSpecialService(@RequestBody UpdateSpecialServiceInVO updateSpecialServiceInVO) {
        specialServiceService.updateSpecialService(updateSpecialServiceInVO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }

    @AspectLog(description = "删除特勤", operationType = BaseEnum.OperationTypeEnum.DELETE)
    @ApiOperation(value = "删除特勤", notes = "删除特勤", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/deleteSpecialService",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject deleteSpecialService(@RequestBody DeleteSpecialServiceInVO deleteSpecialServiceInVO) {
        Integer id = deleteSpecialServiceInVO.getId();
        specialServiceService.deleteSpecialService(id);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }

    @AspectLog(description = "更新路口", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "更新路口", notes = "更新路口", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/updateCross",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject updateCross(@RequestBody UpdateCrossInVO updateCrossInVO) {
        specialServiceService.updateCross(updateCrossInVO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }

    @AspectLog(description = "删除路口", operationType = BaseEnum.OperationTypeEnum.DELETE)
    @ApiOperation(value = "删除路口", notes = "删除路口", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/deleteCross",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject deleteCross(@RequestBody DeleteCrossInVO deleteCrossInVO) {
        specialServiceService.deleteCross(deleteCrossInVO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }


    @AspectLog(description = "路口进口列表", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @ApiOperation(value = "路口进口列表", notes = "路口进口列表", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/crossInList",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject crossInList(@RequestBody CrossIdVO crossIdVO) {
        String crossId = crossIdVO.getCrossId();
        Set<String> crossInList = specialServiceService.crossInList(crossId);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(crossInList);
    }
}
